@@ -252,7 +252,7 @@ id_cmp(int *id1, int *id2)
 static list *
 cond_append(list *l, int *id)
 {
-	if (!list_find(l, id, (fcmp) &id_cmp))
+	if (*id >= 2000 && !list_find(l, id, (fcmp) &id_cmp))
 		 list_append(l, id);
 	return l;
 }
@@ -874,7 +874,7 @@ stmt_join(sql_allocator *sa, stmt *op1, stmt *op2, comp_type cmptype)
 	s->op1 = op1;
 	s->op2 = op2;
 	s->flag = cmptype;
-	s->key = op1->key;
+	s->key = 0;
 	s->nrcols = 2;
 	return s;
 }
